#copyright ReportLab Inc. 2000-2019
#see license.txt for license details
Version='3.5.27'

__version__=Version
__date__='20191001'
import sys

if sys.version_info[0:2]!=(2, 7) and sys.version_info<(3, 5):
    raise ImportError("""rlextra requires Python 2.7+ or 3.5+; 3.0-3.4 are not supported.
If you want to try with other python versions edit line 10 of rlextra/__init__
to remove this error.""")

import reportlab
if reportlab.Version < Version:
	raise ImportError("""rlextra version %s requires at least reportlab %s, and you have reportlab %s; it is strongly recommended that you use the same versions""" % (Version, Version.rsplit('.',1)[0]+'.0', reportlab.Version))
